  function editmask(grid_file,coast_file);

%% An interactive tool for manual editing of ROMS Land/Sea mask defined at
%% RHO-points.  The arguments are
%%
%% "grid_file"  is the ROMS grid netCDF file containing the 2D arrays of lon-
%%              lat coordinates and land/sea mask (all three at RHO points);
%%
%% "coast_file" is either (i) name of a user-supplied .mat file containing
%%              lon-lat coordinates coast line edges; or (ii) "ijcoast.mat"
%%              coast line in (I,J) grid point coordinates [typically saved
%%              from a previous session operating on the same grid]; or
%%              (iii) a single-letter [c, l, i, h, f] to select the desired
%%              resolution among GSHHS coastline datasets.
%%
%% To accelerate the processing, the mask is edited in (I,J) grid coordinates.
%% If coast line in the (I,J) coordinates are not provided, this program will
%% compute them and write them into a file named "ijcoast.mat". If called
%% without, one or both arguments, it will prompt for the needed file names.
%%
%%  mouse shortcuts:
%%
%%          double click ==> Zoom in
%%          right  click ==> Zoom out
%%          middle click ==> change editing mode
%%
%% calls: axisscroll button radiobox textbox
%%
%% Initially written by Andrey Shcherbina, ashcherbina@apl.washington.edu
%%                       in 2001 when he was a student in UCSD Scripps OI
%%
%% This particular branch of editmask is maintained by Alexander Shchepetkin,
%% old_galaxy@yahoo.com, and is modified beyond compatibility with the original
%% version (incl. the ones from AGRIF/CROCO and Rutgers ROMS branches), so this
%% file and all the other programs in this directory may not substitute or be
%% substituted with files with the same names from a different ROMS branch.
%% However, the netCDF ROMS grid files are compatible and this tool (taken as
%% the whole set) may be applied to edit AGRIF/CROCO/Rutgers grid files.
%%
%% 08/20/2016: modified to visualize and edit automatic changes to mask array
%% produced by single_connect --show.

%% 07/30/2015: modified to use native netCDF support instead of legacy mexnc;

%% 05/22/2013: modified to optimize appearance and size of the working
%% window and GUI

% Define and initialize persistent variables.

persistent changed rmask mask_bak xcst ycst Lp Mp mx my
persistent fname_bak fig zooming xx yy xl yl hplot

global GUI   %% single global variable to pass info to/from callback routines

FIGURE_NAME='Land/Sea Mask Editor';

%% Set colormap: Note that after executing "single_connect --show" land mask
%% may have 4 possible values: -2,-1, 0,(land),+1(water). They are mapped into
%% 4 pairs of colors: purple, red, orange(land), and blue(water) in such a way
%% that areas of uniform value are filled with slight checker-board pattern to
%% show the actual grid boxes.
%%
%%       magenta           red - dark-red      yellow-orange       blue
%%   |  narrow bays    | inaccessible lakes |       land     |     water    |
%%    ----------------- -------------------- ---------------- --------------
CMAP=[.5 0 .5; .7 0 .7 ;   1 0 0 ; .7 0 0   ; 1 .7 0; 1 .8 0 ; 0 .9 1; 0 1 1];

LineColor='k';  LineWidth=1;          %%% Set  coastline color and line width.


%% Check whether input grid file name is given: if no argument is present
%% open dialog box and ask user to select ROMS grid file.  This may happen
%% during the first call only, where the meaning of argument "grid_file" is,
%% indeed file name.  During the subsequent calls to "editmask" (initiated
%% from within, that is recursively) the meaning of the first argument is a
%% character string from uicontrols to be used as input argument for the
%% switch just below, while the actual filename is saved as "fname_bak".
%%  For proper operation of the switch argument grid_file must be assigned
%% (not be empty), therefore the dialog box needs to be called here rather
%% than just before reading the file. Coast line file name (if not specified)
%% will be via similar dialog box just before reading the file and does not
%% need to be saved between the calls.

if (nargin < 1 | isempty(grid_file))
  [fn,pth]=uigetfile('*.nc', 'Select ROMS grid file...');
  if (~fn)
    return
  end
  grid_file=[pth,fn];
end
                                 %%% MAIN SWITCH (the rest of this program is
switch lower(grid_file)          %%%  just set of entries within this switch)
  case 'zoomin'                                 %%% ZOOM IN
    disable_click;
    editmask move;
    zooming=1;
    waitforbuttonpress
    xx0=xx; yy0=yy;                             % save current pointer
    rbbox;                                      % position track rubberband
    xx1=xx; yy1=yy;                             % rectangle new pointer
    if ((xx0 ~= xx1) & (yy0 ~= yy1))            % position trim limits and
      xx0(xx0 < 0)=0; xx0(xx0 > xl(2))=xl(2);   % set zoom
      xx1(xx1 < 0)=0; xx1(xx1 > xl(2))=xl(2);
      yy0(yy0 < 0)=0; yy0(yy0 > yl(2))=yl(2);
      yy1(yy1 < 0)=0; yy1(yy1 > yl(2))=yl(2);

      xlim([min([xx0 xx1]), max([xx0 xx1])]);
      ylim([min([yy0 yy1]), max([yy0 yy1])]);
      axisscroll;
    end
    enable_click;
    zooming=0;

  case 'zoomout'                                  %%% ZOOM OUT
    xlim(xl);  ylim(yl);
    axisscroll;

  case 'click'                                    %%% EDIT LAND-SEA MASK
    button=get(gcf, 'SelectionType');
    if (strcmp(button,'alt'))                     % zoom out on right click
      editmask zoomout;
      return;
    end
    if (strcmp(button,'open'))                    % zoom in on double click
      editmask zoomin;
      return;
    end
    if (strcmp(button,'extend'))                  % cycle modes on middle click
      m=mod(GUI.mode,3)+1;
      eval(get(GUI.mode_h(m),'callback'));
      editmask zoomout;
      return;
    end
    if (within(xx,xlim) & within(yy,ylim))        % left click within edit area
      disable_click;
      switch GUI.tool
        case 1                                    % point edit
          ix=floor(xx+1.5);
          iy=floor(yy+1.5);
          switch GUI.mode
            case 1                                % toggle between land and sea
              rmask(ix,iy)=~rmask(ix,iy);
            case 2                                % set land
              rmask(ix,iy)=0;
            case 3                                % set sea
              rmask(ix,iy)=1;
	  end
        case 2                                    % area edit: save current
          xx0=xx; yy0=yy;                         % pointer position track
          rbbox;                                  % rubberband rectangle
          xx1=xx; yy1=yy;                         % new pointer position
          idsel=find((mx-xx0-1).*(mx-xx1-1)<=0 & ...
                     (my-yy0-1).*(my-yy1-1)<=0);  % indices within rectangle
          switch GUI.mode
            case 1
              rmask(idsel)=~rmask(idsel);         % toggle between land and sea
            case 2
              rmask(idsel)=0;                     % set land
            case 3
              rmask(idsel)=1;                     % set sea
          end
      end
      changed=1;
      enable_click;
      editmask refresh;
    end

  case 'refresh'                                     %%% UPDATE image by
    cdata=rmask*2+mod(mod(mx,2)+mod(my,2),2)+5;      %%% changing color map
    set(hplot,'cdata',cdata');
    nm=[FIGURE_NAME,' - ', fname_bak];
    if (changed),
      nm=[nm,'*'];
    end,
    set(fig,'name',nm);

  case 'move'                              %%% POINTER MOVEMENT: update
    xy=get(gca,'currentpoint');            %%% pointer coordinates on menu
    xx=xy(1,1); yy=xy(1,2);
    if (within(xx,xlim) & within(yy,ylim)),
      s=sprintf('[%d,%d]',floor(xx+.5),floor(yy+.5));
      if (zooming),
        pointer('zoom');
      elseif (GUI.tool==1),
%%        pointer('point');
        pointer('crosshair');
      elseif GUI.tool==2,
        pointer('rect');
      end,
    else,
      s='---';
      pointer('arrow');
    end,
    set(GUI.pos_h,'string',s);

  case 'save'                              %% Write out into GRID netCDF file.
    mask_bak=rmask;
    ncid=netcdf.open(fname_bak,'WRITE');
    varid=netcdf.inqVarID(ncid,'mask_rho');
    netcdf.putVar(ncid, varid, mask_bak);
    netcdf.close(ncid);
    disp(['wrote mask_rho into ' fname_bak])
    changed=0;
    editmask refresh;

  case 'undo'                 %%% UNDO CHANGES: restore last saved mask
    rmask=mask_bak;
    changed=0;
    editmask refresh;

  case 'exit'                 %%% EXIT: inquire to save mask
    if (~changed),            %%%  changes if not saved yet
      delete(gcf);
    else
      res=questdlg('The mask has been changed. Save?',FIGURE_NAME);
      switch res
        case 'Yes'
          editmask save;
          disp('Mask has been saved');
          delete(gcf);
        case 'No'
          disp('Mask has NOT been saved');
          delete(gcf);
      end
    end

%% case 'help'                  %%% HELP (Not implemented yet.)
%%   show_help;
                                %%% INITIALIZE: read mask and coastline data
  otherwise
    delete(findobj(0,'tag','maskeditor'));   %%% close all windows.

    tic;                                     %%% Read in ROMS grid file: Note
     ncid=netcdf.open(grid_file,'NOWRITE');  %%% that array "mask_bak" is used
    varid=netcdf.inqVarID(ncid,'mask_rho');  %%% throughout the entire program
     mask_bak=netcdf.getVar(ncid,varid);     %%% both as a I/O buffer and as a
    varid=netcdf.inqVarID(ncid, 'lon_rho');  %%% backup copy so it is possible
     rlon=netcdf.getVar(ncid,varid);         %%% to undo the changes.
    varid=netcdf.inqVarID(ncid, 'lat_rho');  %%% Writing it back into netCDF
     rlat=netcdf.getVar(ncid,varid);         %%% file is done in the reverse
    netcdf.close(ncid);                      %%% order: array "rmask" is copied
    fname_bak=grid_file; rmask=mask_bak;     %%% into "mask_bak" first, then
    [Lp,Mp]=size(mask_bak);                  %%% the latter is written into
    toc;                                     %%%  the netCDF file.

    disp([ 'reading ' grid_file, ' complete, Lp=' num2str(Lp) ...
                                           ' Mm=' num2str(Mp) ])


%% Read in coastline and convert it into "fractional index" coordinate.
%% There are three possibilities:
%%    (1) coast_file is "ijcoast.mat" which already contains precomputed
%%        coastline in index coordinates for the given grid (usually saved
%%        from the previous editing session for the same grid) ==> no
%%        conversion is needed ;
%%    (2) coast_file is user-supplied coastline in lon-lat format;
%%    (3) coast_file not a real file, but is just a key specifying
%%        resolution of GSHHS dataset;

    if (nargin < 2 | isempty(coast_file))
      [fn,pth]=uigetfile('*.mat','Select Coastline file...');
      if (~fn)
        return
      end
      coast_file=[pth,fn];
    elseif (coast_file=='c' | coast_file=='l' | coast_file=='i' ...
                            | coast_file=='h' | coast_file=='f')
      res=coast_file ;
      coast_file=make_coast(res,rlon,rlat);
    end

%% Read in coastline data and convert it into grid (I,J)-continuous-index
%% coordinates (unless reading ijcoast-prepared file which already contains it
%% in this form); Besides the (I,J)-form, the other two formats are acceptable:
%% "lon-lat" (typically comes from legacy user-supplied Matlab-generated .mat
%% files), or in "ncst"-format as extracted from GSHHS data by m_map tools
%% resulting in an array of coastline segments packed as indexed (lon,lat)
%% pairs.

    load(coast_file);
    if (exist('Icst') & exist('Jcst'))
      disp(['read ' coast_file ': already in (I,J)-coordinates']);
      xcst=Icst; ycst=Jcst;
      clear Jcst Icst;

    elseif (exist('lon') & exist('lat'))
      disp(['read ' coast_file ': coastline in (lon,lat)-coordinates']);
      [xcst ycst]=ijcoast(rlon,rlat, lon,lat);
      clear lat lon;

    elseif (exist('ncst'))
      disp(['read ' coast_file ': coastline in GSHHS ncst-format']);
      lon=ncst(:,1); ; lat=ncst(:,2);
      [xcst ycst]=ijcoast(rlon,rlat, lon,lat);
      clear lat lon ncst;

    else
      error(['### ERROR: File ' coast_file ' is incompatible with this ' ...
             'program.  A correct file should contain one of the three ' ...
             'sets of variables: either xcst,ycst; or lon,lat; or ncst.' ])
    end
    disp(['length of coastline ' num2str(size(xcst)) ' points'])
    clear rlon rlat;

% Initialize the window.

    fig=figure('NumberTitle','off',  'tag','maskeditor',...
              'DoubleBuffer','on',  'backingstore','off',...
              'menubar','none');

% Since starting with version 2013a Matlab no longer allows resizing figure
% by window manager, the following segment is to set optimal sizes by Matlab
% itself.  The working portion of the figure sized [wdth * hgth] which are
% set to be proportionally to the dimensions of the grid (thus always keeping
% the aspect ratio) and a specified portion of the screen size; after that
% left, right, bottom, and top side margins dL,dR,dB,dT are added to
% accommodate axes and menu buttons on the right side. These are specified
% in pixels and are kept constant regardless of figure size.

    dL=50; dR=120;   dB=45; dT=25;

    set(0,'Units','pixels')
    screen=get(0,'ScreenSize');

    hgth=0.9*screen(4);       wdth=hgth*Lp/Mp;

    full_width=wdth+dL+dR;    full_height=hgth+dB+dT;

%   set(fig,'Position',[screen(3)-full_width-10 screen(4)-full_height-40 ...
%                                                  full_width full_height]);

    set(fig,'Position',[16 screen(4)-full_height-40 full_width full_height]);

    [mx,my]=ndgrid(1:Lp,1:Mp);  gx=mx(:,1)-1;   gy=my(1,:)-1;

% Possible values rmask= -2, -1, 0, 1  ; cdata = 1, 2, ... , 8

    cdata=rmask*2+mod(mod(mx,2)+mod(my,2),2)+5;

    axes('position', [dL/full_width     dB/full_height ...
                      wdth/full_width   hgth/full_height]);

%    hplot=image(gx,gy,cdata, 'cdatamapping','direct', ...
%                                'erasemode','normal');

%    set(gca, 'YDir','normal', 'drawmode','fast', ...
%              'layer','top',  'tickdir','out');


% Matlab 2014b and later complains about erasemode and drawmode,

    hplot=image(gx,gy,cdata, 'cdatamapping','direct');

    set(gca, 'YDir','normal', 'layer','top', 'tickdir','out');



    colormap(CMAP);
    hold on;
    hline=plot(xcst,ycst,LineColor);
    set(hline,'LineWidth',LineWidth);
    xl=xlim; yl=ylim;
    changed=0;
    setgui(full_width,wdth,dR,dL);
    editmask refresh;
end
return

                                                 %% set-up Land/Sea mask
function setgui(full_width,wdth,dR,dL)           %% editing menu bottons.

  xpos=(dL+wdth+.1*dR)/full_width;   bwdth=0.85*dR/full_width;

  textbox([xpos .85  bwdth .10], '(i,j)', {'0,0'}, 'pos');

  radiobox([xpos .625 bwdth .19], ...
         'Edit Mode',{'Toggle Land/Sea','Set Land','Set Sea'},'mode');

  radiobox([xpos .45 bwdth .14], ...
            'Edit Tool',{'Point edit','Area edit'},'tool');

  button([xpos .375 bwdth .05], 'Zoom In',[mfilename ' zoomin']);

  button([xpos .325 bwdth .05], 'Zoom Out',[mfilename ' zoomout']);

  button([xpos .225 bwdth .05], 'Undo',[mfilename ' undo']);

  button([xpos .175 bwdth .05], 'Save',[mfilename ' save']);

  button([xpos .075 bwdth .05], 'Brexit',[mfilename ' exit']);

  axisscroll('r')
  axisscroll('t')
  set(gcf,'WindowButtonMotionFcn',[mfilename ' move;'],...
          'CloseRequestFcn',[mfilename ' exit;'],...
          'interruptible','on');
  enable_click;
return


function disable_click                                   %% Disable pointer
 set(gcf,'WindowButtonDownFcn','');                      %% clicking on
return                                                   %% current figure.

function enable_click                                    %% Enable pointer
 set(gcf,'WindowButtonDownFcn',[mfilename ' click;']);   %% clicking on
return                                                   %% current figure.

function r=within(a,b)                         %% Check if "a" is within
 r=(a >= b(1) & a <=  b(2));                   %% the range of '[b(1),b(2)]
return
