function fname = make_coast(res, rlon,rlat)

%% Read GSHHS dataset and create as smaller, more manageable file to be used
%% for the particular grid. Basically this program takes 2D coordinate arrays
%% rlon,rlat and computes their minimum and maximum values, therefore forming
%% a logically rectangular window.  Then it reads GSHHS dataset (via "m_map",
%% a package from Rich Pawlowicz, https://www.eoas.ubc.ca/~rich/map.html) and
%% extracts the relevant part of global coast line saving it into a .mat file.
%%
%% Argument "res" selects GSHHS dataset:
%% res='c'  gshhs_c.b    crude resolution data
%%     'l'  gshhs_l.b    low resolution data
%%     'i'  gshhs_i.b    intermediate resolution data
%%     'h'  gshhs_h.b    high resolution data
%%     'f'  gshhs_f.b    full resolution data
%%
%% Output "fname" is the name of .mat file which can be read by editmask.
%% This program 100% relies on m_map functionality to access files in GSHHS
%% format (not readable by Matlab directly), extract relevant data, and write
%% it into m_map own Matlab-readable format (file fname below) which is then
%% further converted to editmask-readable .mat file.
%%
%% Further Information: http://www.brest.ird.fr/Roms_tools/
%%
%% This file is part of ROMSTOOLS
%%
%% ROMSTOOLS is free software; you can redistribute it and/or modify it under
%% the terms of GNU General Public License as published by the Free Software
%% Foundation; either version 2 of the License, or (at your option) any later
%% version.
%%
%% ROMSTOOLS is distributed in the hope that it will be useful, but WITHOUT
%% ANY WARRANTY,  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%% You should have received a copy of the GNU General Public License along
%% with this program; if not, write to the Free Software Foundation, Inc.,
%% 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%
%% Copyright (c) 2002-2006 by Pierrick Penven, Pierrick.Penven@ird.fr
%%
%% Updated    13-Sep-2005 by Patrick Marchesiello (IRD)

lonmin=min(min(rlon));  lonmax=max(max(rlon));
latmin=min(min(rlat));  latmax=max(max(rlat));

dl=.1*max(lonmax-lonmin,latmax-latmin);

xmin=lonmin-dl;  xmax=lonmax+dl;
ymin=latmin-dl;  ymax=latmax+dl;

fname=['coastline_' res '.mat'];
disp(['make_coast :: extracting GSHHS into ' fname ' ...'])
m_proj('mercator', 'lon',[xmin xmax], 'lat',[ymin ymax]);

tic;
if res=='c'
  m_gshhs_c('save',fname);
elseif res=='l'
  m_gshhs_l('save',fname);
elseif res=='i'
  m_gshhs_i('save',fname);
elseif res=='h'
  m_gshhs_h('save',fname);
elseif res=='f'
  m_gshhs_f('save',fname);
end
toc;
disp(['make_coast :: making plot ...'])
tic;
m_usercoast(fname,'patch', [.9 .9 .9]);
m_grid('box','fancy','tickdir','in');
toc;

%% The last part is to convert "ncst"-formatted coastline file into
%% a more traditional "lon-lat" form.  It is no longer needed because
%% editmask.m has been modified to accept the m_map package-generated
%% "ncst"-format file directly. 

%% Save coastline data in lon,lat form to be used in editmask
%% tic;
%% cstfname=['coastline_',res,'_mask.mat'];
%% disp(['make_coast :: saving ', cstfname, ' ...'])
%% load(fname)
%% lon=ncst(:,1); ; lat=ncst(:,2);
%% disp(['Saving ',cstfname,'...'])
%% eval(['save ', cstfname, ' lon lat']);
%% clear lat lon ncst ;
%% toc;
